/***************************************************************************
 *   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSSERIAL_INTERFACE_H
#define CONEXUSSERIAL_INTERFACE_H

#include <conexus/pointer.h>

namespace Conexus
{

  /**
   * @author Rick L Vinyard Jr
   */
  class SerialInterface
  {
    public:

      typedef ConexusPointer<SerialInterface> pointer;
      
      SerialInterface() { }

      virtual ~SerialInterface() { }
      
      /** Opens the named serial port device in the specified mode. */
      virtual void open( const std::string& device, ReadWrite rwmode = READ_WRITE_UNCHANGED ) = 0;

      /**
       * Will attempt to open the serial port if a device has been specified. Otherwise an
       * exception will be thrown.
       */
      virtual void open() = 0;

      virtual uint32_t rw_mode() = 0;

      /**
       * Will return the string representing the device associated with this serial object.
       */
      virtual std::string device() = 0;

      /**
       * Sets the string representing the device associated with this serial object.
       *
       * If the serial device is currently open and d is not equal to the current device,
       * the current device will be closed.
       *
       * Also, if auto_open is set, the new device will be immediately opened.
       *
       * Additionally, this method checks whether the parameter d is the same as the current device,
       * and parameter mode is the same as the current mode.
       * If both are equal, the device will not be closed and reopened unless force=true.
       */
      virtual void set_device( const std::string& device, ReadWrite rwmode = READ_WRITE_UNCHANGED ) = 0;
      
      virtual sigc::signal<void>& signal_device_changed() = 0;
      
  };

}

#endif
