/***************************************************************************
 *   Copyright (C) 2009 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the conexus library.                             *
 *                                                                         *
 *   The conexus library is free software; you can redistribute it and/or  *
 *   modify it under the terms of the GNU General Public License           *
 *   version 3 as published by the Free Software Foundation.               *
 *                                                                         *
 *   The conexus library is distributed in the hope that it will be        *
 *   useful, but WITHOUT ANY WARRANTY; without even the implied warranty   *
 *   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU   *
 *   General Public License for more details.                              *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this software. If not see <http://www.gnu.org/licenses/>.  *
 ***************************************************************************/
#ifndef CONEXUSTIME_H
#define CONEXUSTIME_H

#include <sys/time.h>
#include <cstdint>
#include <sigc++/sigc++.h>

namespace Conexus
{

  /**
    @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
  */
  class TimeVal
  {
    public:
      TimeVal ( int64_t seconds=0, int64_t microseconds=0 );
      TimeVal ( const struct timeval& tv );
      TimeVal ( const TimeVal& other );

      ~TimeVal();

      int64_t seconds() const;
      int64_t microseconds() const;

      void set_seconds ( int64_t seconds );
      void set_microseconds ( int64_t microseconds );
      void set ( int64_t seconds, int64_t microseconds );

      TimeVal& operator= ( const struct timeval& tv );

      void clear();

      bool set_to_current_time();

      void add ( const TimeVal& other );
      void subtract ( const TimeVal& other );
      void add_seconds ( int64_t seconds );
      void subtract_seconds ( int64_t seconds );
      void add_milliseconds ( int64_t milliseconds );
      void subtract_milliseconds ( int64_t milliseconds );
      void add_microseconds ( int64_t microseconds );
      void subtract_microsecond ( int64_t microseconds );

      TimeVal  operator+ ( const TimeVal& other ) const;
      TimeVal  operator- ( const TimeVal& other ) const;
      TimeVal  operator+ ( int64_t seconds ) const;
      TimeVal  operator- ( int64_t seconds ) const;
      TimeVal& operator+= ( const TimeVal& other );
      TimeVal& operator-= ( const TimeVal& other );
      TimeVal& operator+= ( int64_t seconds );
      TimeVal& operator-= ( int64_t seconds );

      float as_float() const;
      double as_double() const;
      long double as_long_double() const;

      operator struct timeval() const;
      operator float() const;
      operator double() const;
      operator long double() const;

      bool negative() const;

    protected:
      int64_t m_seconds;
      int64_t m_microseconds;

      void cleanup();

  };

}

#endif
